@@ -8,9 +8,9 @@ use warnings;
 use Scalar::Util qw(blessed);
 
 use Sub::Exporter -setup => {
-	exports => [qw(obj inv obj_can inv_can)],
+	exports => [qw(obj obj_does inv inv_does obj_can inv_can)],
 	groups => {
-		default => [qw(obj inv)],
+		default => [qw(obj obj_does inv)],
 	},
 };
 
@@ -18,7 +18,7 @@ use constant CAN_HAS_DOES => not not UNIVERSAL->can("DOES");
 
 use warnings::register;
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 sub extract_io {
 	my $glob = shift;
@@ -47,6 +47,20 @@ sub extract_io {
 sub obj ($;$); # predeclare, it's recursive
 
 sub obj ($;$) {
+	my ( $object_or_filehandle, $class ) = @_;
+
+	my $object = blessed($object_or_filehandle)
+		? $object_or_filehandle
+		: extract_io($object_or_filehandle) || return;
+
+	if ( defined $class ) {
+		$object->isa($class)
+	} else {
+		return 1; # return $object? what if it's overloaded?
+	}
+}
+
+sub obj_does ($;$) {
 	my ( $object_or_filehandle, $class_or_role ) = @_;
 
 	my $object = blessed($object_or_filehandle)
@@ -54,9 +68,13 @@ sub obj ($;$) {
 		: extract_io($object_or_filehandle) || return;
 
 	if ( defined $class_or_role ) {
-		return CAN_HAS_DOES
-			? $object->DOES($class_or_role)
-			: $object->isa($class_or_role)
+		if ( CAN_HAS_DOES ) {
+			# we can be faster in 5.10
+			$object->DOES($class_or_role);
+		} else {
+			my $method = $object->can("DOES") || "isa";
+			$object->$method($class_or_role);
+		}
 	} else {
 		return 1; # return $object? what if it's overloaded?
 	}
@@ -66,7 +84,7 @@ sub inv ($;$) {
 	my ( $inv, $class_or_role ) = @_;
 
 	if ( blessed($inv) ) {
-		return obj($inv, $class_or_role);
+		return obj_does($inv, $class_or_role);
 	} else {
 		# we check just for scalar keys on the stash because:
 		# sub Foo::Bar::gorch {}
@@ -74,11 +92,24 @@ sub inv ($;$) {
 		# Bar->can("isa") # false
 		# this means that 'Foo' is a valid invocant, but Bar is not
 
-		if ( !ref($inv) and defined $inv and length($inv) and do { no strict 'refs'; scalar keys %{$inv . "::"} } ) {
+		if ( !ref($inv)
+				and
+			defined $inv
+				and
+			length($inv)
+				and
+			do { no strict 'refs'; scalar keys %{$inv . "::"} }
+		) {
+			# it's considered a class name as far as gv_fetchmethod is concerned
+			# even if the class def is empty
 			if ( defined $class_or_role ) {
-				return CAN_HAS_DOES
-					? $inv->DOES($class_or_role)
-					: $inv->isa($class_or_role)
+				if ( CAN_HAS_DOES ) {
+					# we can be faster in 5.10
+					$inv->DOES($class_or_role);
+				} else {
+					my $method = $inv->can("DOES") || "isa";
+					$inv->$method($class_or_role);
+				}
 			} else {
 				return 1; # $inv is always true, so not a problem, but that would be inconsistent
 			}
@@ -133,24 +164,40 @@ an object, and if so asking about its class.
 
 =over 4
 
-=item obj $thing, [ $class_or_role ]
+=item obj $thing, [ $class ]
 
 This function tests if C<$thing> is an object.
 
-If C<$class_or_role> is provided, it also tests tests whether
-C<< $thing->DOES($class_or_role) >>.
+If C<$class> is provided, it also tests tests whether
+C<< $thing->isa($class) >>.
+
+C<$thing> is considered an object if it's blessed, or if it's a C<GLOB> with a
+valid C<IO> slot (the C<IO> slot contains a L<FileHandle> object which is the
+actual invocant). This corresponds directly to C<gv_fetchmethod>.
+
+=item obj_does $thing, [ $class_or_role ]
+
+Just like C<obj> but uses L<UNIVERSAL/DOES> instead of L<UNIVERSAL/isa>.
 
 L<UNIVERSAL/DOES> is just like C<isa>, except it's use is encouraged to query
 about an interface, as opposed to the object structure. If C<DOES> is not
 overridden by th ebject, calling it is semantically identical to calling
 C<isa>.
 
-C<$thing> is considered an object if it's blessed, or if it's a C<GLOB> with a
-valid C<IO> slot (this is a L<FileHandle> object).
+This is probably reccomended over C<obj> for interoperability, but can be
+slower on Perls before 5.10.
+
+Note that L<UNIVERSAL/DOES>
 
 =item inv $thing, [ $class_or_role ]
 
-Just like C<obj>, but also returns true for classes.
+Just like C<obj_does>, but also returns true for classes.
+
+Note that this method is slower, but is supposed to return true for any value
+you can call methods on (class, object, filehandle, etc).
+
+Look into L<autobox> if you would like to be able to call methods on all
+values.
 
 =item obj_can $thing, $method
 
@@ -161,6 +208,10 @@ appropriate.
 
 =back
 
+=head1 SEE ALSO
+
+L<UNIVERSAL>, L<Params::Util>, L<autobox>, L<Moose>, L<asa>
+
 =head1 VERSION CONTROL
 
 This module is maintained using Darcs. You can get the latest version from
